@@ -17,10 +17,10 @@ build_time_vars = {'ABIFLAGS': 'm',
  'BINDIR': '/usr/local/bin',
  'BINLIBDEST': '/usr/local/lib/python3.3',
  'BLDLIBRARY': '-L. -lpython3.3m',
- 'BLDSHARED': 'cc -pthread -shared -fPIC -L/usr/local/lib/ -L/usr/obj/ports/Python-3.3.2/Python-3.3.2',
+ 'BLDSHARED': 'cc -pthread -shared -fPIC -L/usr/local/lib/ -L/usr/obj/ports/Python-3.3.5/Python-3.3.5',
  'BUILDEXE': '',
  'BUILDPYTHON': 'python',
- 'BUILD_GNU_TYPE': 'sh-unknown-openbsd5.5',
+ 'BUILD_GNU_TYPE': 'mips64-unknown-openbsd5.6',
  'BYTESTR_DEPS': '\\',
  'CC': 'cc -pthread',
  'CCSHARED': '-fPIC',
@@ -28,13 +28,13 @@ build_time_vars = {'ABIFLAGS': 'm',
  'CFLAGSFORSHARED': '-fPIC',
  'CONFIGFILES': 'configure configure.ac acconfig.h pyconfig.h.in Makefile.pre.in',
  'CONFIGURE_CFLAGS': '-O2 -pipe',
- 'CONFIGURE_CPPFLAGS': '',
+ 'CONFIGURE_CPPFLAGS': '-I/usr/local/include',
  'CONFIGURE_LDFLAGS': '-L/usr/local/lib/',
- 'CONFIG_ARGS': "'--enable-shared' '--srcdir=/usr/obj/ports/Python-3.3.2/Python-3.3.2' '--with-fpectl' '--with-threads' '--enable-ipv6' '--with-cxx_main' '--with-system-expat' '--with-system-ffi' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' 'CC=cc' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib/'",
+ 'CONFIG_ARGS': "'--enable-shared' '--srcdir=/usr/obj/ports/Python-3.3.5/Python-3.3.5' '--with-fpectl' '--with-threads' '--enable-ipv6' '--with-cxx_main' '--with-system-expat' '--with-system-ffi' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' 'CC=cc' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib/' 'CPPFLAGS=-I/usr/local/include'",
  'CONFINCLUDEDIR': '/usr/local/include',
  'CONFINCLUDEPY': '/usr/local/include/python3.3m',
  'COREPYTHONPATH': ':plat-openbsd5',
- 'CPPFLAGS': '-I. -IInclude -I./Include',
+ 'CPPFLAGS': '-I. -IInclude -I./Include -I/usr/local/include',
  'CXX': 'c++ -pthread',
  'DESTDIRS': '/usr/local /usr/local/lib /usr/local/lib/python3.3 /usr/local/lib/python3.3/lib-dynload',
  'DESTLIB': '/usr/local/lib/python3.3',
@@ -47,8 +47,8 @@ build_time_vars = {'ABIFLAGS': 'm',
  'DLINCLDIR': '.',
  'DLLLIBRARY': '',
  'DOUBLE_IS_ARM_MIXED_ENDIAN_IEEE754': 0,
- 'DOUBLE_IS_BIG_ENDIAN_IEEE754': 0,
- 'DOUBLE_IS_LITTLE_ENDIAN_IEEE754': 1,
+ 'DOUBLE_IS_BIG_ENDIAN_IEEE754': 1,
+ 'DOUBLE_IS_LITTLE_ENDIAN_IEEE754': 0,
  'DYNLOADFILE': 'dynload_shlib.o',
  'ENABLE_IPV6': 1,
  'EXE': '',
@@ -70,6 +70,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_ACOSH': 1,
  'HAVE_ADDRINFO': 1,
  'HAVE_ALARM': 1,
+ 'HAVE_ALLOCA_H': 0,
  'HAVE_ALTZONE': 0,
  'HAVE_ASINH': 1,
  'HAVE_ASM_TYPES_H': 0,
@@ -155,7 +156,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_GAMMA': 1,
  'HAVE_GCC_ASM_FOR_X64': 0,
  'HAVE_GCC_ASM_FOR_X87': 0,
- 'HAVE_GCC_UINT128_T': 0,
+ 'HAVE_GCC_UINT128_T': 1,
  'HAVE_GETADDRINFO': 1,
  'HAVE_GETCWD': 1,
  'HAVE_GETC_UNLOCKED': 1,
@@ -202,7 +203,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_KILLPG': 1,
  'HAVE_KQUEUE': 1,
  'HAVE_LANGINFO_H': 1,
- 'HAVE_LARGEFILE_SUPPORT': 1,
+ 'HAVE_LARGEFILE_SUPPORT': 0,
  'HAVE_LCHFLAGS': 0,
  'HAVE_LCHMOD': 0,
  'HAVE_LCHOWN': 1,
@@ -210,7 +211,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_LIBDL': 0,
  'HAVE_LIBDLD': 0,
  'HAVE_LIBIEEE': 0,
- 'HAVE_LIBINTL_H': 0,
+ 'HAVE_LIBINTL_H': 1,
  'HAVE_LIBREADLINE': 1,
  'HAVE_LIBRESOLV': 0,
  'HAVE_LIBSENDFILE': 0,
@@ -260,6 +261,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_PREAD': 1,
  'HAVE_PROCESS_H': 0,
  'HAVE_PROTOTYPES': 1,
+ 'HAVE_PTHREAD_ATFORK': 1,
  'HAVE_PTHREAD_DESTRUCTOR': 0,
  'HAVE_PTHREAD_H': 1,
  'HAVE_PTHREAD_INIT': 0,
@@ -426,7 +428,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HGBRANCH': '',
  'HGTAG': '',
  'HGVERSION': '',
- 'HOST_GNU_TYPE': 'sh-unknown-openbsd5.5',
+ 'HOST_GNU_TYPE': 'mips64-unknown-openbsd5.6',
  'INCLDIRSTOMAKE': '/usr/local/include /usr/local/include /usr/local/include/python3.3m /usr/local/include/python3.3m',
  'INCLUDEDIR': '/usr/local/include',
  'INCLUDEPY': '/usr/local/include/python3.3m',
@@ -439,11 +441,11 @@ build_time_vars = {'ABIFLAGS': 'm',
  'IO_H': 'Modules/_io/_iomodule.h',
  'IO_OBJS': '\\',
  'LDCXXSHARED': 'c++ -pthread -shared -fPIC',
- 'LDFLAGS': '-L/usr/local/lib/ -L/usr/obj/ports/Python-3.3.2/Python-3.3.2',
+ 'LDFLAGS': '-L/usr/local/lib/ -L/usr/obj/ports/Python-3.3.5/Python-3.3.5',
  'LDLAST': '',
  'LDLIBRARY': 'libpython3.3m.so',
  'LDLIBRARYDIR': '',
- 'LDSHARED': 'cc -pthread -shared -fPIC -L/usr/local/lib/ -L/usr/obj/ports/Python-3.3.2/Python-3.3.2',
+ 'LDSHARED': 'cc -pthread -shared -fPIC -L/usr/local/lib/ -L/usr/obj/ports/Python-3.3.5/Python-3.3.5',
  'LDVERSION': '3.3m',
  'LIBC': '',
  'LIBDEST': '/usr/local/lib/python3.3',
@@ -520,11 +522,11 @@ build_time_vars = {'ABIFLAGS': 'm',
  'PYTHON_HEADERS': '\\',
  'PYTHON_OBJS': '\\',
  'PY_CFLAGS': '-fno-strict-aliasing -DNDEBUG -O2 -pipe  -fPIC -O2 -pipe -O2 -pipe',
- 'PY_CORE_CFLAGS': '-fno-strict-aliasing -DNDEBUG -O2 -pipe  -fPIC -O2 -pipe -O2 -pipe -I. -IInclude -I./Include -fPIC -DPy_BUILD_CORE',
- 'PY_CPPFLAGS': '-I. -IInclude -I./Include',
+ 'PY_CORE_CFLAGS': '-fno-strict-aliasing -DNDEBUG -O2 -pipe  -fPIC -O2 -pipe -O2 -pipe -I. -IInclude -I./Include -I/usr/local/include -fPIC -DPy_BUILD_CORE',
+ 'PY_CPPFLAGS': '-I. -IInclude -I./Include -I/usr/local/include',
  'PY_FORMAT_LONG_LONG': '"ll"',
  'PY_FORMAT_SIZE_T': '"z"',
- 'PY_LDFLAGS': '-L/usr/local/lib/ -L/usr/obj/ports/Python-3.3.2/Python-3.3.2',
+ 'PY_LDFLAGS': '-L/usr/local/lib/ -L/usr/obj/ports/Python-3.3.5/Python-3.3.5',
  'Py_DEBUG': 0,
  'Py_ENABLE_SHARED': 1,
  'QUICKTESTOPTS': '-x test_subprocess test_io test_lib2to3 \\',
@@ -532,7 +534,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'READELF': 'readelf',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.3.2/Python-3.3.2:',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.3.5/Python-3.3.5:',
  'SCRIPTDIR': '/usr/local/lib',
  'SETPGRP_HAVE_ARG': 1,
  'SGI_ABI': '',
@@ -546,17 +548,17 @@ build_time_vars = {'ABIFLAGS': 'm',
  'SIZEOF_FLOAT': 4,
  'SIZEOF_FPOS_T': 8,
  'SIZEOF_INT': 4,
- 'SIZEOF_LONG': 4,
- 'SIZEOF_LONG_DOUBLE': 8,
+ 'SIZEOF_LONG': 8,
+ 'SIZEOF_LONG_DOUBLE': 16,
  'SIZEOF_LONG_LONG': 8,
  'SIZEOF_OFF_T': 8,
  'SIZEOF_PID_T': 4,
- 'SIZEOF_PTHREAD_T': 4,
+ 'SIZEOF_PTHREAD_T': 8,
  'SIZEOF_SHORT': 2,
- 'SIZEOF_SIZE_T': 4,
+ 'SIZEOF_SIZE_T': 8,
  'SIZEOF_TIME_T': 8,
- 'SIZEOF_UINTPTR_T': 4,
- 'SIZEOF_VOID_P': 4,
+ 'SIZEOF_UINTPTR_T': 8,
+ 'SIZEOF_VOID_P': 8,
  'SIZEOF_WCHAR_T': 4,
  'SIZEOF__BOOL': 1,
  'SO': '.so',
@@ -570,11 +572,13 @@ build_time_vars = {'ABIFLAGS': 'm',
  'SYSLIBS': '-lm',
  'SYS_SELECT_WITH_SYS_TIME': 1,
  'TANH_PRESERVES_ZERO_SIGN': 1,
+ 'TCLTK_INCLUDES': '',
+ 'TCLTK_LIBS': '',
  'TESTOPTS': '',
  'TESTPATH': '',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.3.2/Python-3.3.2: ./python',
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.3.5/Python-3.3.5: ./python',
  'TESTPYTHONOPTS': '',
- 'TESTRUNNER': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.3.2/Python-3.3.2: ./python ./Tools/scripts/run_tests.py',
+ 'TESTRUNNER': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.3.5/Python-3.3.5: ./python ./Tools/scripts/run_tests.py',
  'TESTTIMEOUT': 3600,
  'THREADOBJ': 'Python/thread.o',
  'TIMEMODULE_LIB': 0,
@@ -598,8 +602,8 @@ build_time_vars = {'ABIFLAGS': 'm',
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/usr/obj/ports/Python-3.3.2/Python-3.3.2',
- 'abs_srcdir': '/usr/obj/ports/Python-3.3.2/Python-3.3.2',
+ 'abs_builddir': '/usr/obj/ports/Python-3.3.5/Python-3.3.5',
+ 'abs_srcdir': '/usr/obj/ports/Python-3.3.5/Python-3.3.5',
  'datarootdir': '/usr/local/share',
  'exec_prefix': '/usr/local',
  'prefix': '/usr/local',
